"""
    an image processing technique to produce a 1D signal for each pixel : this 1D signal is obtained by projecting the neighborhood of this pixel on a set of bases (each base is used as a convilution kernel)
    https://subversion.ipr.univ-rennes1.fr/repos/main/projects/antipode/src/python/antipode/texori.py
"""
from imageengine import IImageEngine, PixelType

class IProjectorBase(object):
    """
        a set of projectors

        a projector is a grey level image, in which each pixel value is acting as a weight
    """
    def __init__(self):
        pass

    def get_num_projectors(self):
        pass
        
    def get_projector_kernel(self, projector_index):

        pass

def create_circle_image(image_size, circle_radius, circle_pos, circle_thickness, background_value=0.0, circle_value=1.0):
    """
    :param dict image_size:
    """
    ie = IImageEngine.get_instance()
    width = image_size['width']
    height = image_size['height']
    circle_pos_x = float(circle_pos['x'])
    circle_pos_y = float(circle_pos['y'])
    r_min = circle_radius - circle_thickness * 0.5
    assert r_min >= 0.0
    r_max = circle_radius + circle_thickness * 0.5
    r_min_square = r_min * r_min
    r_max_square = r_max * r_max
    image = ie.create_image(width=width, height=height, pixel_type=PixelType.F32)
    for y in range(height):
        for x in range(width):
            dx = x - circle_pos_x
            dy = y - circle_pos_y
            r_square = (dx * dx + dy * dy)
            if r_min_square < r_square < r_max_square:
                pixel_value = circle_value
            else:
                pixel_value = background_value
            image.set_pixel(x, y, pixel_value)
    return image

class CircularSymetryProjectorBase(IProjectorBase):
    """
        generates a base of circles 
    """

    def __init__(self, max_radius, oversampling_scale=2):
        """
        :param int max_radius: the biggest circle radius in the set of projectors
        :param int oversampling_scale: oversampling is used to generate antialased circles. The higher this value, the better the quality of antialiasing is
        """
        super().__init__()
        assert max_radius > 0
        assert oversampling_scale > 0
        self.max_radius = max_radius
        self.oversampling_scale = oversampling_scale
        self.num_projectors = max_radius + 1

    def get_num_projectors(self):
        return self.num_projectors
        
    def get_projector_kernel(self, projector_index):
        assert projector_index < self.get_num_projectors()
        radius = projector_index
        image_size = self.max_radius * 2 + 1
        circle_pos = {'x': self.max_radius, 'y': self.max_radius}
        oversampled_circle = create_circle_image(
            image_size={'width':image_size * self.oversampling_scale, 'height':image_size*self.oversampling_scale},
            circle_radius=radius * self.oversampling_scale,
            circle_pos={'x': self.max_radius* self.oversampling_scale, 'y': self.max_radius* self.oversampling_scale},
            circle_thickness=1 * self.oversampling_scale)
        circle_image = oversampled_circle.resample(width=image_size, height=image_size)
        anchor_point = circle_pos
        return circle_image, anchor_point


# class LocalProjector:
#     """ This image processor computes the probability for each pixel to have a material oriented in the given direction
    
#     The technique is based on the notion of a projector. The role of the projector is to project (or cumulate) the 2D neighborhood of each pixel on a projection axis that passes through the pixel and that has an orientation chosen by the user.

#     """


#     def __init__(self, searched_orientation, image_process_listener=imageprocessing.NullImageProcessListener()):
#         imageprocessing.IImageProcessor.__init__(self, image_process_listener)
#         self.searched_orientation = searched_orientation
#         self.orientation_amount_image = None

#     @staticmethod
#     def compute_noise_in_perpendicular_orientation(src_image, orientation, half_window_width=5, image_process_listener=imageprocessing.NullImageProcessListener()):
#         """ Computes an image that gives for each pixel, the amount of noise in a direction perpendicular to the given orientation

#         Because the projection has to be performed for each pixel, we encode the projector operator in the form of a set of num_projectors 2D kernels that are used as a filter. Each kernel cumulates the image signal (therefore computes the sum of the image signal) along a line perpendicular to the projection axis, but each at a different position (offset) on this projection axis.
  
#           :param src_image: the input image, the one we want to process
#           :type src_image: numpy.array
          
#           :param orientation: the orientation along which the noise needs to be computed
#           :type orientation: float (in radians)
          
#           :param half_window_width: half size of the neighborhood window, in pixels (the heighborood window is a window of size (2*half_window_width+1, 2*half_window_width+1) centered on each pixel). The bigger, the more precise the measurement but with less locality.
#         """
#         projector_base = OrientationProbabilityEstimator.ProjectorBase2(orientation, searched_segment_size=10.0, cut_blur_stddev=3.0, image_process_listener=image_process_listener)
#         projections = numpy.empty((src_image.shape[0], src_image.shape[1], projector_base.get_num_projectors()), dtype=numpy.float32)
#         # print('projections.shape=%s' % str(projections.shape))
#         for projector_index in range(projector_base.get_num_projectors()):
#             projector, center_of_filter = projector_base.get_projector_kernel(projector_index)

#             projection = cv2.filter2D(src_image.astype(numpy.float32), ddepth=-1, kernel=projector, anchor=tuple(center_of_filter))
#             image_process_listener.onImage(projection, 'projection_%f_%d' % (orientation, projector_index))
#             projections[:, :, projector_index] = projection
#         # for each pixel, compute the gradient along its projection axis
#         # swap axes because I don't think the filter2D only works along fist 2 axes
#         projections = numpy.swapaxes(projections, 1, 2)
#         grad_kernel = numpy.array((-0.5, 0.5), dtype=numpy.float32)
#         gradients = cv2.filter2D(projections, ddepth=-1, kernel=grad_kernel, anchor=(0, 0))

#         if image_process_listener is not None:
#             # grab a slice of the gradient image for debugging purpose
#             cut = gradients[:, :, gradients.shape[2] / 2]
#             image_process_listener.onImage(cut, 'cut_%f' % (orientation))

#         gradients = numpy.swapaxes(gradients, 1, 2)
#         gradients *= gradients
#         texture_amount = numpy.sum(gradients, axis=2)
#         image_process_listener.onImage(texture_amount, 'texture_amount_%f' % (orientation))
#         return texture_amount

#     def processImage(self, image):
#         self.orientation_amount_image = OrientationProbabilityEstimator.compute_noise_in_perpendicular_orientation(image, self.searched_orientation, self.get_image_process_listener())

#     def get_result(self):
#         return self.orientation_amount_image
    